//---------------------------------------------------------------------------------
/// \file	SimpleCommand.cpp
/// \author	Rob Bateman [robthebloke@hotmail.com]
/// \date	2-7-2004
/// \brief	This is the simplest possible example of how to write a mel command
///			that can be added into Maya via a plugin. You can call the command 
///			from mel by typing, "SimpleCommand". This simply executes a mel call
///			via MGlobal::executeCommand to create a sphere.
///
///			mayald -shared -o computation.so simple_main.cpp
///
//---------------------------------------------------------------------------------

// to compile:
//
//		mayald -shared -o computation.so main.cpp
//

#ifdef WIN32
	#define NT_PLUGIN
#endif


// required to link the libraries
#ifdef WIN32
	#pragma comment(lib,"Foundation.lib")
	#pragma comment(lib,"OpenMaya.lib")
	#pragma comment(lib,"OpenMayaFx.lib")
	#pragma comment(lib,"OpenMayaUi.lib")
	#pragma comment(lib,"Image.lib")
	#pragma comment(lib,"OpenMayaAnim.lib")
#endif


// include any maya headers we need.
#include <maya/MSimple.h>
#include <maya/MGlobal.h>
#include <maya/MMatrix.h>
#include <maya/MFnMesh.h>
#include <maya/MFnTransform.h>
#include <maya/MItDependencyNodes.h>
#include <maya/MPointArray.h>
#include <maya/MComputation.h>

//---------------------------------------------------------------------------------
// Use a helper macro to register a command with Maya.  This creates and
// registers a command that does not support undo or redo.  The
// created class derives off of MPxCommand.
//
DeclareSimpleCommand( computation, "A simple mel function example", "1.0");


//---------------------------------------------------------------------------------
/// \brief	builds a world space transform for a shape by walking up the transform hierarchy
/// \param	obj	-	the handle to the transform node
/// \return	the world space transform matrix for the object
///
MMatrix BuildTransform(const MObject& obj)
{
	MFnTransform fn(obj);

	// get matrix transform for this bone
	MMatrix M = fn.transformationMatrix();

	if (fn.parentCount()>0) {
		MObject _obj =fn.parent(0);
		return M* BuildTransform(_obj) ;
	}
	return M;
}


//---------------------------------------------------------------------------------
/// \brief	This function impliments our very first simple command plugin.
/// \param	args - the argument list that was passes to the command from MEL
/// \return	MS::kSuccess - command succeeded
///			MS::kFailure - command failed (returning this value will cause the
///         	           MEL script that is being run to terminate unless the
///         	           error is caught using a "catch" statement.
///
MStatus computation::doIt( const MArgList& args )

{
	// create an MComputation class for heavy calculation type things
	MComputation Interupter;

	// set the start of the heavy computation
	Interupter.beginComputation();

		// now do all of your computation safe in the knowledge you can press esapce
		// at any time (here, i'm just looping from frame 0 to 1000, and writing all
		// points out to a binary file).
		//
		for(unsigned int i=0;i!=1000;++i) {

			// sets the current frame
			MGlobal::viewFrame(i);

			/// generate a filename with frame padding
			char filename[256];
			sprintf(filename,"/cgstaff/rbateman/temp/file.%4d.dat",i);

			/// replace spaces with zeros
			for(unsigned int i=0;filename[i];++i) {
				if(filename[i] == ' ') {
					filename[i] = '0';
				}
			}

			/// open the binary file
			FILE* fp = fopen(filename,"wb");

			// loop through all mesh nodes
			MItDependencyNodes it(MFn::kMesh);
			while( !it.isDone() )
			{
				// attach a function set to the mesh
				MFnMesh fnMesh( it.item() );

				// retrieve the mesh points (local space)
				MPointArray points;
				fnMesh.getPoints( points );

				// loop through each instance of the mesh. (ie, the number of
				// parent transforms the mesh has)
				//
				for(int j=0;j!=fnMesh.parentCount();++j) {

					MMatrix parent_matrix = BuildTransform( fnMesh.parent(j) );

					for(int k=0;k!=points.length() ; ++k)
					{
						// transform the points from local space to world space
						//
						MPoint OutPoint = points[k] * parent_matrix;

						// write 3D vector to file
						float data[3];
						data[0] = OutPoint.x;
						data[1] = OutPoint.y;
						data[2] = OutPoint.z;

						fwrite(data,sizeof(float),3,fp);


						//------------------------------------------------------------
						// ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **
						// check the computation to see if it's been interupted!

						if( Interupter.isInterruptRequested() )
						{
							fclose(fp);
							return MS::kFailure;
						}

						// ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **
						//------------------------------------------------------------

					}
				}

				// got next mesh
				it.next();
			}

			// close the file
			fclose(fp);
		}

	// finished our computation
	Interupter.endComputation();

	return MS::kSuccess;
}













































